[INFRA] Import NVIDIA/CCCL upstream as optimization reference library
CCCL (CUDA C++ Core Libraries) provides: - CUB: device/block/warp-level GPU primitives (reduce, scan, sort, topk) - Thrust: high-level parallel algorithms (transform_reduce, sort, scan) - libcudacxx: CUDA C++ standard library (atomics, barriers, memory) - cudax: experimental features (memory resources, allocators) - Tuning policies: per-SM hardware-specific algorithm parameters Competition optimization vectors mapped to CCCL: - Output TPS (83% weight): warp_reduce, block_reduce, device_topk - Input TPS (14% weight): device_scan, block_load, prefetch - Cache TPS (3% weight): prefix caching strategy patterns - Memory (0.9 util): pooled/cached/buddy allocators Source: https://github.com/NVIDIA/cccl (shallow clone, HEAD only) License: Apache-2.0
This commit is contained in:
95
cccl_upstream/cub/benchmarks/bench/reduce/arg_extrema.cu
Normal file
95
cccl_upstream/cub/benchmarks/bench/reduce/arg_extrema.cu
Normal file
@@ -0,0 +1,95 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2026, NVIDIA CORPORATION. All rights reserved.
|
||||
// SPDX-License-Identifier: BSD-3-Clause
|
||||
|
||||
#include <cub/device/device_reduce.cuh>
|
||||
#include <cub/device/dispatch/tuning/tuning_reduce.cuh>
|
||||
|
||||
#include <cuda/std/type_traits>
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS_PER_THREAD ipt 7:24:1
|
||||
// %RANGE% TUNE_THREADS_PER_BLOCK tpb 128:1024:32
|
||||
// %RANGE% TUNE_ITEMS_PER_VEC_LOAD_POW2 ipv 1:2:1
|
||||
|
||||
#if !TUNE_BASE
|
||||
struct tuned_policy_selector
|
||||
{
|
||||
[[nodiscard]] _CCCL_HOST_DEVICE constexpr auto operator()(cuda::compute_capability) const -> cub::ReducePolicy
|
||||
{
|
||||
cub::ReducePassPolicy rp{
|
||||
TUNE_THREADS_PER_BLOCK,
|
||||
TUNE_ITEMS_PER_THREAD,
|
||||
1 << TUNE_ITEMS_PER_VEC_LOAD_POW2,
|
||||
cub::BLOCK_REDUCE_WARP_REDUCTIONS,
|
||||
cub::LOAD_DEFAULT};
|
||||
return {rp, rp};
|
||||
}
|
||||
};
|
||||
#endif // !TUNE_BASE
|
||||
|
||||
template <typename T, typename OpT>
|
||||
void arg_reduce(nvbench::state& state, nvbench::type_list<T, OpT>)
|
||||
{
|
||||
// Offset type used to index within the total input in the range [d_in, d_in + num_items)
|
||||
using offset_t = cuda::std::int64_t;
|
||||
|
||||
// Retrieve axis parameters
|
||||
const auto elements = static_cast<std::size_t>(state.get_int64("Elements{io}"));
|
||||
thrust::device_vector<T> in = generate(elements);
|
||||
thrust::device_vector<offset_t> out_index(1);
|
||||
thrust::device_vector<T> out_extremum(1);
|
||||
|
||||
const T* d_in = thrust::raw_pointer_cast(in.data());
|
||||
offset_t* d_out_index = thrust::raw_pointer_cast(out_index.data());
|
||||
T* d_out_extremum = thrust::raw_pointer_cast(out_extremum.data());
|
||||
|
||||
// Enable throughput calculations and add "Size" column to results.
|
||||
state.add_element_count(elements);
|
||||
state.add_global_memory_reads<T>(elements, "Size");
|
||||
state.add_global_memory_writes<offset_t>(1);
|
||||
state.add_global_memory_writes<T>(1);
|
||||
|
||||
caching_allocator_t alloc;
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch& launch) {
|
||||
auto env = cub_bench_env(
|
||||
alloc,
|
||||
launch
|
||||
#if !TUNE_BASE
|
||||
,
|
||||
cuda::execution::tune(tuned_policy_selector{})
|
||||
#endif // !TUNE_BASE
|
||||
);
|
||||
if constexpr (cuda::std::is_same_v<OpT, cub::detail::arg_min>)
|
||||
{
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::ArgMin,
|
||||
"ArgMin failed",
|
||||
d_in,
|
||||
d_out_extremum,
|
||||
d_out_index,
|
||||
static_cast<offset_t>(elements),
|
||||
cuda::std::less{},
|
||||
env);
|
||||
}
|
||||
else
|
||||
{
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::ArgMax,
|
||||
"ArgMax failed",
|
||||
d_in,
|
||||
d_out_extremum,
|
||||
d_out_index,
|
||||
static_cast<offset_t>(elements),
|
||||
cuda::std::less{},
|
||||
env);
|
||||
}
|
||||
});
|
||||
}
|
||||
|
||||
using op_types = nvbench::type_list<cub::detail::arg_min, cub::detail::arg_max>;
|
||||
|
||||
NVBENCH_BENCH_TYPES(arg_reduce, NVBENCH_TYPE_AXES(fundamental_types, op_types))
|
||||
.set_name("base")
|
||||
.set_type_axes_names({"T{ct}", "Operation{ct}"})
|
||||
.add_int64_power_of_two_axis("Elements{io}", nvbench::range(16, 28, 4));
|
||||
69
cccl_upstream/cub/benchmarks/bench/reduce/base.cuh
Normal file
69
cccl_upstream/cub/benchmarks/bench/reduce/base.cuh
Normal file
@@ -0,0 +1,69 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2011-2026, NVIDIA CORPORATION. All rights reserved.
|
||||
// SPDX-License-Identifier: BSD-3
|
||||
|
||||
#pragma once
|
||||
|
||||
#include <cub/device/device_reduce.cuh>
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
#if !TUNE_BASE
|
||||
template <typename AccumT>
|
||||
struct policy_selector
|
||||
{
|
||||
[[nodiscard]] _CCCL_HOST_DEVICE constexpr auto operator()(cuda::compute_capability) const -> cub::ReducePolicy
|
||||
{
|
||||
const auto [items, threads] =
|
||||
cub::detail::scale_mem_bound(TUNE_THREADS_PER_BLOCK, TUNE_ITEMS_PER_THREAD, int{sizeof(AccumT)});
|
||||
const auto policy = cub::ReducePassPolicy{
|
||||
threads, items, 1 << TUNE_ITEMS_PER_VEC_LOAD_POW2, cub::BLOCK_REDUCE_WARP_REDUCTIONS, cub::LOAD_DEFAULT};
|
||||
return {policy, policy};
|
||||
}
|
||||
};
|
||||
#endif // !TUNE_BASE
|
||||
|
||||
template <typename T, typename OffsetT>
|
||||
void reduce(nvbench::state& state, nvbench::type_list<T, OffsetT>)
|
||||
{
|
||||
using init_value_t = T;
|
||||
|
||||
// Retrieve axis parameters
|
||||
const auto elements = state.get_int64("Elements{io}");
|
||||
|
||||
thrust::device_vector<T> in = generate(elements);
|
||||
thrust::device_vector<T> out(1);
|
||||
|
||||
auto d_in = thrust::raw_pointer_cast(in.data());
|
||||
auto d_out = thrust::raw_pointer_cast(out.data());
|
||||
|
||||
// Enable throughput calculations and add "Size" column to results.
|
||||
state.add_element_count(elements);
|
||||
state.add_global_memory_reads<T>(elements, "Size");
|
||||
state.add_global_memory_writes<T>(1);
|
||||
|
||||
caching_allocator_t alloc;
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch& launch) {
|
||||
auto env = cub_bench_env(
|
||||
alloc,
|
||||
launch
|
||||
#if !TUNE_BASE
|
||||
,
|
||||
cuda::execution::tune(policy_selector<cuda::std::__accumulator_t<op_t, T, init_value_t>>{})
|
||||
#endif // !TUNE_BASE
|
||||
);
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::Reduce,
|
||||
"Reduce failed",
|
||||
d_in,
|
||||
d_out,
|
||||
static_cast<OffsetT>(elements),
|
||||
op_t{},
|
||||
init_value_t{},
|
||||
env);
|
||||
});
|
||||
}
|
||||
|
||||
NVBENCH_BENCH_TYPES(reduce, NVBENCH_TYPE_AXES(value_types, offset_types))
|
||||
.set_name("base")
|
||||
.set_type_axes_names({"T{ct}", "OffsetT{ct}"})
|
||||
.add_int64_power_of_two_axis("Elements{io}", nvbench::range(16, 28, 4));
|
||||
132
cccl_upstream/cub/benchmarks/bench/reduce/by_key.cu
Normal file
132
cccl_upstream/cub/benchmarks/bench/reduce/by_key.cu
Normal file
@@ -0,0 +1,132 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2011-2023, NVIDIA CORPORATION. All rights reserved.
|
||||
// SPDX-License-Identifier: BSD-3
|
||||
|
||||
#include <cub/device/device_reduce.cuh>
|
||||
|
||||
#include <look_back_helper.cuh>
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS ipt 7:24:1
|
||||
// %RANGE% TUNE_THREADS tpb 128:1024:32
|
||||
// %RANGE% TUNE_TRANSPOSE trp 0:1:1
|
||||
// %RANGE% TUNE_LOAD ld 0:1:1
|
||||
// %RANGE% TUNE_MAGIC_NS ns 0:2048:4
|
||||
// %RANGE% TUNE_DELAY_CONSTRUCTOR_ID dcid 0:7:1
|
||||
// %RANGE% TUNE_L2_WRITE_LATENCY_NS l2w 0:1200:5
|
||||
|
||||
#if !TUNE_BASE
|
||||
struct bench_reduce_by_key_policy_selector
|
||||
{
|
||||
[[nodiscard]] _CCCL_HOST_DEVICE constexpr auto operator()(cuda::compute_capability) const -> cub::ReduceByKeyPolicy
|
||||
{
|
||||
return {
|
||||
cub::ReduceByKeyAlgorithm::lookback,
|
||||
{
|
||||
TUNE_THREADS,
|
||||
TUNE_ITEMS,
|
||||
TUNE_TRANSPOSE == 0 ? cub::BLOCK_LOAD_DIRECT : cub::BLOCK_LOAD_WARP_TRANSPOSE,
|
||||
TUNE_LOAD == 0 ? cub::LOAD_DEFAULT : cub::LOAD_CA,
|
||||
cub::BLOCK_SCAN_WARP_SCANS,
|
||||
lookback_delay_policy,
|
||||
},
|
||||
};
|
||||
}
|
||||
};
|
||||
#endif // !TUNE_BASE
|
||||
|
||||
template <class KeyT, class ValueT, class OffsetT>
|
||||
static void reduce_by_key(nvbench::state& state, nvbench::type_list<KeyT, ValueT, OffsetT>)
|
||||
{
|
||||
using reduction_op_t = ::cuda::std::plus<>;
|
||||
|
||||
const auto elements = static_cast<std::size_t>(state.get_int64("Elements{io}"));
|
||||
constexpr std::size_t min_segment_size = 1;
|
||||
const std::size_t max_segment_size = static_cast<std::size_t>(state.get_int64("MaxSegSize"));
|
||||
|
||||
thrust::device_vector<OffsetT> num_runs_out(1);
|
||||
thrust::device_vector<ValueT> in_vals(elements);
|
||||
thrust::device_vector<ValueT> out_vals(elements);
|
||||
thrust::device_vector<KeyT> out_keys(elements);
|
||||
thrust::device_vector<KeyT> in_keys = generate.uniform.key_segments(elements, min_segment_size, max_segment_size);
|
||||
|
||||
const KeyT* d_in_keys = thrust::raw_pointer_cast(in_keys.data());
|
||||
KeyT* d_out_keys = thrust::raw_pointer_cast(out_keys.data());
|
||||
const ValueT* d_in_vals = thrust::raw_pointer_cast(in_vals.data());
|
||||
ValueT* d_out_vals = thrust::raw_pointer_cast(out_vals.data());
|
||||
OffsetT* d_num_runs_out = thrust::raw_pointer_cast(num_runs_out.data());
|
||||
|
||||
caching_allocator_t alloc;
|
||||
|
||||
// Run once to get the number of runs for reporting
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::ReduceByKey,
|
||||
"ReduceByKey failed",
|
||||
d_in_keys,
|
||||
d_out_keys,
|
||||
d_in_vals,
|
||||
d_out_vals,
|
||||
d_num_runs_out,
|
||||
reduction_op_t{},
|
||||
static_cast<OffsetT>(elements),
|
||||
alloc);
|
||||
cudaDeviceSynchronize();
|
||||
const OffsetT num_runs = num_runs_out[0];
|
||||
|
||||
state.add_element_count(elements);
|
||||
state.add_global_memory_reads<KeyT>(elements);
|
||||
state.add_global_memory_reads<ValueT>(elements);
|
||||
state.add_global_memory_writes<ValueT>(num_runs);
|
||||
state.add_global_memory_writes<KeyT>(num_runs);
|
||||
state.add_global_memory_writes<OffsetT>(1);
|
||||
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch& launch) {
|
||||
auto env = cub_bench_env(
|
||||
alloc,
|
||||
launch
|
||||
#if !TUNE_BASE
|
||||
,
|
||||
cuda::execution::tune(bench_reduce_by_key_policy_selector{})
|
||||
#endif // !TUNE_BASE
|
||||
);
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::ReduceByKey,
|
||||
"ReduceByKey failed",
|
||||
d_in_keys,
|
||||
d_out_keys,
|
||||
d_in_vals,
|
||||
d_out_vals,
|
||||
d_num_runs_out,
|
||||
reduction_op_t{},
|
||||
static_cast<OffsetT>(elements),
|
||||
env);
|
||||
});
|
||||
}
|
||||
|
||||
using some_offset_types = nvbench::type_list<nvbench::int32_t>;
|
||||
|
||||
#ifdef TUNE_KeyT
|
||||
using key_types = nvbench::type_list<TUNE_KeyT>;
|
||||
#else // !defined(TUNE_KeyT)
|
||||
using key_types =
|
||||
nvbench::type_list<int8_t,
|
||||
int16_t,
|
||||
int32_t,
|
||||
int64_t
|
||||
# if _CCCL_HAS_INT128()
|
||||
,
|
||||
int128_t
|
||||
# endif
|
||||
>;
|
||||
#endif // TUNE_KeyT
|
||||
|
||||
#ifdef TUNE_ValueT
|
||||
using value_types = nvbench::type_list<TUNE_ValueT>;
|
||||
#else // !defined(TUNE_ValueT)
|
||||
using value_types = all_types;
|
||||
#endif // TUNE_ValueT
|
||||
|
||||
NVBENCH_BENCH_TYPES(reduce_by_key, NVBENCH_TYPE_AXES(key_types, value_types, some_offset_types))
|
||||
.set_name("base")
|
||||
.set_type_axes_names({"KeyT{ct}", "ValueT{ct}", "OffsetT{ct}"})
|
||||
.add_int64_power_of_two_axis("Elements{io}", nvbench::range(16, 28, 4))
|
||||
.add_int64_power_of_two_axis("MaxSegSize", {1, 4, 8});
|
||||
14
cccl_upstream/cub/benchmarks/bench/reduce/custom.cu
Normal file
14
cccl_upstream/cub/benchmarks/bench/reduce/custom.cu
Normal file
@@ -0,0 +1,14 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2011-2023, NVIDIA CORPORATION. All rights reserved.
|
||||
// SPDX-License-Identifier: BSD-3
|
||||
|
||||
// This benchmark uses a custom reduction operation, max_t, which is not known to CUB, so no operator specific
|
||||
// optimizations (e.g. using redux or DPX instructions) are performed. This benchmark covers the unoptimized code path.
|
||||
|
||||
// Because CUB cannot detect this operator, we cannot add any tunings based on the results of this benchmark. Its main
|
||||
// use is to detect regressions.
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
using value_types = all_types;
|
||||
using op_t = max_t;
|
||||
#include "base.cuh"
|
||||
@@ -0,0 +1,93 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2026, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
|
||||
#include <cub/device/device_reduce.cuh>
|
||||
|
||||
#include <thrust/detail/raw_pointer_cast.h>
|
||||
#include <thrust/device_vector.h>
|
||||
|
||||
#include <cuda/argument>
|
||||
#include <cuda/execution.determinism.h>
|
||||
#include <cuda/execution.require.h>
|
||||
#include <cuda/std/functional>
|
||||
#include <cuda/std/utility>
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
#include <nvbench/range.cuh>
|
||||
#include <nvbench/types.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS_PER_THREAD ipt 3:24:1
|
||||
// %RANGE% TUNE_THREADS_PER_BLOCK tpb 128:1024:32
|
||||
|
||||
#if !TUNE_BASE
|
||||
struct policy_selector_t
|
||||
{
|
||||
[[nodiscard]] _CCCL_HOST_DEVICE constexpr auto operator()(cuda::compute_capability) const -> cub::ReducePolicy
|
||||
{
|
||||
const auto p = cub::ReducePassPolicy{
|
||||
TUNE_THREADS_PER_BLOCK, TUNE_ITEMS_PER_THREAD, 1, cub::BLOCK_REDUCE_RAKING, cub::LOAD_DEFAULT};
|
||||
return {p, p};
|
||||
}
|
||||
};
|
||||
#endif // !TUNE_BASE
|
||||
|
||||
template <class T, class OffsetT>
|
||||
void deterministic_sum(nvbench::state& state, nvbench::type_list<T, OffsetT>)
|
||||
try
|
||||
{
|
||||
using init_value_t = T;
|
||||
|
||||
if (!cuda::std::in_range<OffsetT>(state.get_int64("Elements{io}")))
|
||||
{
|
||||
state.skip("Skipping: Elements{io} is not representable by OffsetT.");
|
||||
return;
|
||||
}
|
||||
const auto elements = static_cast<OffsetT>(state.get_int64("Elements{io}"));
|
||||
|
||||
thrust::device_vector<T> in = generate(elements);
|
||||
thrust::device_vector<T> out(1, thrust::no_init);
|
||||
thrust::device_vector<OffsetT> device_num_items{elements};
|
||||
|
||||
auto d_in = thrust::raw_pointer_cast(in.data());
|
||||
auto d_out = thrust::raw_pointer_cast(out.data());
|
||||
auto d_num_items = thrust::raw_pointer_cast(device_num_items.data());
|
||||
|
||||
// Enable throughput calculations and add "Size" column to results.
|
||||
state.add_element_count(elements);
|
||||
state.add_global_memory_reads<T>(elements, "Size");
|
||||
state.add_global_memory_writes<T>(1);
|
||||
|
||||
caching_allocator_t alloc;
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch& launch) {
|
||||
auto env = cub_bench_env(
|
||||
alloc,
|
||||
launch,
|
||||
cuda::execution::require(cuda::execution::determinism::gpu_to_gpu)
|
||||
#if !TUNE_BASE
|
||||
,
|
||||
cuda::execution::tune(policy_selector_t{})
|
||||
#endif // !TUNE_BASE
|
||||
);
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::Reduce,
|
||||
"Reduce failed",
|
||||
d_in,
|
||||
d_out,
|
||||
cuda::args::deferred{d_num_items},
|
||||
cuda::std::plus<>{},
|
||||
init_value_t{},
|
||||
env);
|
||||
});
|
||||
}
|
||||
catch (const std::bad_alloc&)
|
||||
{
|
||||
state.skip("Skipping: out of memory.");
|
||||
}
|
||||
|
||||
using types = nvbench::type_list<float, double>;
|
||||
NVBENCH_BENCH_TYPES(deterministic_sum, NVBENCH_TYPE_AXES(types, offset_types))
|
||||
.set_name("base")
|
||||
.set_type_axes_names({"T{ct}", "OffsetT{ct}"})
|
||||
// 2^32 exceeds INT32_MAX to cover the code paths for problem sizes that exceed a single 32-bit chunk
|
||||
.add_int64_power_of_two_axis("Elements{io}", {16, 20, 24, 28, 32});
|
||||
@@ -0,0 +1,98 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2026, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
|
||||
#include <cub/device/device_reduce.cuh>
|
||||
|
||||
#include <thrust/detail/raw_pointer_cast.h>
|
||||
#include <thrust/device_vector.h>
|
||||
|
||||
#include <cuda/argument>
|
||||
#include <cuda/execution.determinism.h>
|
||||
#include <cuda/execution.require.h>
|
||||
#include <cuda/std/functional>
|
||||
|
||||
#include <cstddef>
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
#include <nvbench/range.cuh>
|
||||
#include <nvbench/types.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS_PER_THREAD ipt 3:24:1
|
||||
// %RANGE% TUNE_THREADS_PER_BLOCK tpb 128:1024:32
|
||||
// %RANGE% TUNE_ITEMS_PER_VEC_LOAD_POW2 ipv 1:2:1
|
||||
|
||||
#if !TUNE_BASE
|
||||
template <typename AccumT>
|
||||
struct policy_selector
|
||||
{
|
||||
[[nodiscard]] _CCCL_HOST_DEVICE constexpr auto operator()(cuda::compute_capability) const -> cub::ReducePolicy
|
||||
{
|
||||
const auto [items, threads] =
|
||||
cub::detail::scale_mem_bound(TUNE_THREADS_PER_BLOCK, TUNE_ITEMS_PER_THREAD, int{sizeof(AccumT)});
|
||||
const auto policy = cub::ReducePassPolicy{
|
||||
threads,
|
||||
items,
|
||||
1 << TUNE_ITEMS_PER_VEC_LOAD_POW2,
|
||||
cub::BLOCK_REDUCE_WARP_REDUCTIONS_NONDETERMINISTIC,
|
||||
cub::LOAD_DEFAULT};
|
||||
return {policy, {}};
|
||||
}
|
||||
};
|
||||
#endif // !TUNE_BASE
|
||||
|
||||
template <typename T, typename OffsetT>
|
||||
void nondeterministic_sum(nvbench::state& state, nvbench::type_list<T, OffsetT>)
|
||||
{
|
||||
using op_t = cuda::std::plus<>;
|
||||
using init_value_t = T;
|
||||
|
||||
// Retrieve axis parameters
|
||||
const auto elements = static_cast<std::size_t>(state.get_int64("Elements{io}"));
|
||||
|
||||
thrust::device_vector<T> in = generate(elements);
|
||||
thrust::device_vector<T> out(1, thrust::no_init);
|
||||
thrust::device_vector<OffsetT> device_num_items(1, static_cast<OffsetT>(elements));
|
||||
|
||||
auto d_in = thrust::raw_pointer_cast(in.data());
|
||||
auto d_out = thrust::raw_pointer_cast(out.data());
|
||||
auto d_num_items = thrust::raw_pointer_cast(device_num_items.data());
|
||||
|
||||
// Enable throughput calculations and add "Size" column to results.
|
||||
state.add_element_count(elements);
|
||||
state.add_global_memory_reads<T>(elements, "Size");
|
||||
state.add_global_memory_writes<T>(1);
|
||||
|
||||
caching_allocator_t alloc;
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch& launch) {
|
||||
auto env = cub_bench_env(
|
||||
alloc,
|
||||
launch,
|
||||
cuda::execution::require(cuda::execution::determinism::not_guaranteed)
|
||||
#if !TUNE_BASE
|
||||
,
|
||||
cuda::execution::tune(policy_selector<cuda::std::__accumulator_t<op_t, T, init_value_t>>{})
|
||||
#endif // !TUNE_BASE
|
||||
);
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::Reduce,
|
||||
"Reduce failed",
|
||||
d_in,
|
||||
d_out,
|
||||
cuda::args::deferred{d_num_items},
|
||||
op_t{},
|
||||
init_value_t{},
|
||||
env);
|
||||
});
|
||||
}
|
||||
|
||||
#ifdef TUNE_T
|
||||
using value_types = nvbench::type_list<TUNE_T>;
|
||||
#else
|
||||
using value_types = nvbench::type_list<int32_t, int64_t, float, double>;
|
||||
#endif
|
||||
|
||||
NVBENCH_BENCH_TYPES(nondeterministic_sum, NVBENCH_TYPE_AXES(value_types, offset_types))
|
||||
.set_name("base")
|
||||
.set_type_axes_names({"T{ct}", "OffsetT{ct}"})
|
||||
.add_int64_power_of_two_axis("Elements{io}", nvbench::range(16, 28, 4));
|
||||
86
cccl_upstream/cub/benchmarks/bench/reduce/deferred_sum.cu
Normal file
86
cccl_upstream/cub/benchmarks/bench/reduce/deferred_sum.cu
Normal file
@@ -0,0 +1,86 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2026, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
|
||||
#include <cub/device/device_reduce.cuh>
|
||||
|
||||
#include <thrust/detail/raw_pointer_cast.h>
|
||||
#include <thrust/device_vector.h>
|
||||
|
||||
#include <cuda/argument>
|
||||
#include <cuda/std/functional>
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
#include <nvbench/range.cuh>
|
||||
#include <nvbench/types.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS_PER_THREAD ipt 7:24:1
|
||||
// %RANGE% TUNE_THREADS_PER_BLOCK tpb 128:1024:32
|
||||
// %RANGE% TUNE_ITEMS_PER_VEC_LOAD_POW2 ipv 1:2:1
|
||||
|
||||
#if !TUNE_BASE
|
||||
template <typename AccumT>
|
||||
struct policy_selector
|
||||
{
|
||||
[[nodiscard]] _CCCL_HOST_DEVICE constexpr auto operator()(cuda::compute_capability) const -> cub::ReducePolicy
|
||||
{
|
||||
const auto [items, threads] =
|
||||
cub::detail::scale_mem_bound(TUNE_THREADS_PER_BLOCK, TUNE_ITEMS_PER_THREAD, int{sizeof(AccumT)});
|
||||
const auto policy = cub::ReducePassPolicy{
|
||||
threads, items, 1 << TUNE_ITEMS_PER_VEC_LOAD_POW2, cub::BLOCK_REDUCE_WARP_REDUCTIONS, cub::LOAD_DEFAULT};
|
||||
return {policy, policy};
|
||||
}
|
||||
};
|
||||
#endif // !TUNE_BASE
|
||||
|
||||
using op_t = cuda::std::plus<>;
|
||||
|
||||
template <typename T, typename OffsetT>
|
||||
void reduce(nvbench::state& state, nvbench::type_list<T, OffsetT>)
|
||||
{
|
||||
using init_value_t = T;
|
||||
|
||||
// Retrieve axis parameters
|
||||
const auto elements = state.get_int64("Elements{io}");
|
||||
|
||||
thrust::device_vector<T> in = generate(elements);
|
||||
thrust::device_vector<T> out(1, thrust::default_init);
|
||||
thrust::device_vector<OffsetT> device_num_items(1, static_cast<OffsetT>(elements));
|
||||
|
||||
auto d_in = thrust::raw_pointer_cast(in.data());
|
||||
auto d_out = thrust::raw_pointer_cast(out.data());
|
||||
auto d_num_items = thrust::raw_pointer_cast(device_num_items.data());
|
||||
|
||||
// Enable throughput calculations and add "Size" column to results.
|
||||
state.add_element_count(elements);
|
||||
state.add_global_memory_reads<T>(elements, "Size");
|
||||
state.add_global_memory_writes<T>(1);
|
||||
|
||||
caching_allocator_t alloc;
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch& launch) {
|
||||
auto env = cub_bench_env(
|
||||
alloc,
|
||||
launch
|
||||
#if !TUNE_BASE
|
||||
,
|
||||
cuda::execution::tune(policy_selector<cuda::std::__accumulator_t<op_t, T, init_value_t>>{})
|
||||
#endif // !TUNE_BASE
|
||||
);
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::Reduce,
|
||||
"Reduce failed",
|
||||
d_in,
|
||||
d_out,
|
||||
cuda::args::deferred{d_num_items},
|
||||
op_t{},
|
||||
init_value_t{},
|
||||
env);
|
||||
});
|
||||
}
|
||||
|
||||
using value_types = all_types;
|
||||
|
||||
NVBENCH_BENCH_TYPES(reduce, NVBENCH_TYPE_AXES(value_types, offset_types))
|
||||
.set_name("base")
|
||||
.set_type_axes_names({"T{ct}", "OffsetT{ct}"})
|
||||
.add_int64_power_of_two_axis("Elements{io}", nvbench::range(16, 28, 4));
|
||||
77
cccl_upstream/cub/benchmarks/bench/reduce/deterministic.cu
Normal file
77
cccl_upstream/cub/benchmarks/bench/reduce/deterministic.cu
Normal file
@@ -0,0 +1,77 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2026, NVIDIA CORPORATION. All rights reserved.
|
||||
// SPDX-License-Identifier: BSD-3
|
||||
|
||||
#include <cub/device/device_reduce.cuh>
|
||||
|
||||
#include <cuda/execution.determinism.h>
|
||||
#include <cuda/execution.require.h>
|
||||
#include <cuda/std/utility>
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
#include <nvbench/range.cuh>
|
||||
#include <nvbench/types.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS_PER_THREAD ipt 3:24:1
|
||||
// %RANGE% TUNE_THREADS_PER_BLOCK tpb 128:1024:32
|
||||
|
||||
#if !TUNE_BASE
|
||||
struct policy_selector_t
|
||||
{
|
||||
[[nodiscard]] _CCCL_HOST_DEVICE constexpr auto operator()(cuda::compute_capability) const -> cub::ReducePolicy
|
||||
{
|
||||
const auto p = cub::ReducePassPolicy{
|
||||
TUNE_THREADS_PER_BLOCK, TUNE_ITEMS_PER_THREAD, 1, cub::BLOCK_REDUCE_RAKING, cub::LOAD_DEFAULT};
|
||||
return {p, p};
|
||||
}
|
||||
};
|
||||
#endif // !TUNE_BASE
|
||||
|
||||
template <class T, class OffsetT>
|
||||
void deterministic_sum(nvbench::state& state, nvbench::type_list<T, OffsetT>)
|
||||
try
|
||||
{
|
||||
using init_value_t = T;
|
||||
|
||||
if (!cuda::std::in_range<OffsetT>(state.get_int64("Elements{io}")))
|
||||
{
|
||||
state.skip("Skipping: Elements{io} is not representable by OffsetT.");
|
||||
return;
|
||||
}
|
||||
const auto elements = static_cast<OffsetT>(state.get_int64("Elements{io}"));
|
||||
|
||||
thrust::device_vector<T> in = generate(elements);
|
||||
thrust::device_vector<T> out(1);
|
||||
|
||||
const T* d_in = thrust::raw_pointer_cast(in.data());
|
||||
T* d_out = thrust::raw_pointer_cast(out.data());
|
||||
state.add_element_count(elements);
|
||||
state.add_global_memory_reads<T>(elements, "Size");
|
||||
state.add_global_memory_writes<T>(out.size());
|
||||
|
||||
caching_allocator_t alloc;
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch& launch) {
|
||||
auto env = cub_bench_env(
|
||||
alloc,
|
||||
launch,
|
||||
cuda::execution::require(cuda::execution::determinism::gpu_to_gpu)
|
||||
#if !TUNE_BASE
|
||||
,
|
||||
cuda::execution::tune(policy_selector_t{})
|
||||
#endif // !TUNE_BASE
|
||||
);
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::Reduce, "Reduce failed", d_in, d_out, elements, cuda::std::plus<>{}, init_value_t{}, env);
|
||||
});
|
||||
}
|
||||
catch (const std::bad_alloc&)
|
||||
{
|
||||
state.skip("Skipping: out of memory.");
|
||||
}
|
||||
|
||||
using types = nvbench::type_list<float, double>;
|
||||
NVBENCH_BENCH_TYPES(deterministic_sum, NVBENCH_TYPE_AXES(types, offset_types))
|
||||
.set_name("base")
|
||||
.set_type_axes_names({"T{ct}", "OffsetT{ct}"})
|
||||
// 2^32 exceeds INT32_MAX to cover the code paths for problem sizes that exceed a single 32-bit chunk
|
||||
.add_int64_power_of_two_axis("Elements{io}", {16, 20, 24, 28, 32});
|
||||
36
cccl_upstream/cub/benchmarks/bench/reduce/min.cu
Normal file
36
cccl_upstream/cub/benchmarks/bench/reduce/min.cu
Normal file
@@ -0,0 +1,36 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2011-2023, NVIDIA CORPORATION. All rights reserved.
|
||||
// SPDX-License-Identifier: BSD-3
|
||||
|
||||
// This benchmark is intended to cover DPX instructions on Hopper+ architectures. It specifically uses cuda::minimum<>
|
||||
// instead of a user-defined operator, which CUB recognizes to select an optimized code path.
|
||||
|
||||
// Tuning parameters found for ::cuda::minimum<> apply equally for ::cuda::maximum<>
|
||||
// Tuning parameters found for signed integer types apply equally for unsigned integer types
|
||||
// TODO(bgruber): do tuning parameters found for int16_t apply equally for __half or __nv_bfloat16 on SM90+?
|
||||
|
||||
#include <cuda/functional>
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS_PER_THREAD ipt 7:24:1
|
||||
// %RANGE% TUNE_THREADS_PER_BLOCK tpb 128:1024:32
|
||||
// %RANGE% TUNE_ITEMS_PER_VEC_LOAD_POW2 ipv 1:2:1
|
||||
|
||||
// __half and __nv_bfloat16 are added for full (non-tuning) runs; CUB has fast paths for them (see #9587).
|
||||
#ifdef TUNE_T
|
||||
using value_types = nvbench::type_list<TUNE_T>;
|
||||
#else
|
||||
using value_types =
|
||||
push_back_t<fundamental_types
|
||||
# if _CCCL_HAS_NVFP16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
,
|
||||
__half
|
||||
# endif
|
||||
# if _CCCL_HAS_NVBF16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
,
|
||||
__nv_bfloat16
|
||||
# endif
|
||||
>;
|
||||
#endif
|
||||
using op_t = ::cuda::minimum<>;
|
||||
#include "base.cuh"
|
||||
@@ -0,0 +1,89 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2026, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
|
||||
// SPDX-License-Identifier: BSD-3-Clause
|
||||
|
||||
#include <cub/device/device_reduce.cuh>
|
||||
|
||||
#include <cuda/execution.determinism.h>
|
||||
#include <cuda/execution.require.h>
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
#include <nvbench/range.cuh>
|
||||
#include <nvbench/types.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS_PER_THREAD ipt 3:24:1
|
||||
// %RANGE% TUNE_THREADS_PER_BLOCK tpb 128:1024:32
|
||||
// %RANGE% TUNE_ITEMS_PER_VEC_LOAD_POW2 ipv 1:2:1
|
||||
|
||||
#if !TUNE_BASE
|
||||
template <typename AccumT>
|
||||
struct policy_selector
|
||||
{
|
||||
[[nodiscard]] _CCCL_HOST_DEVICE constexpr auto operator()(cuda::compute_capability) const -> cub::ReducePolicy
|
||||
{
|
||||
const auto [items, threads] =
|
||||
cub::detail::scale_mem_bound(TUNE_THREADS_PER_BLOCK, TUNE_ITEMS_PER_THREAD, int{sizeof(AccumT)});
|
||||
const auto policy = cub::ReducePassPolicy{
|
||||
threads,
|
||||
items,
|
||||
1 << TUNE_ITEMS_PER_VEC_LOAD_POW2,
|
||||
cub::BLOCK_REDUCE_WARP_REDUCTIONS_NONDETERMINISTIC,
|
||||
cub::LOAD_DEFAULT};
|
||||
return {policy, {}};
|
||||
}
|
||||
};
|
||||
#endif // !TUNE_BASE
|
||||
|
||||
template <typename T, typename OffsetT>
|
||||
void nondeterministic_sum(nvbench::state& state, nvbench::type_list<T, OffsetT>)
|
||||
{
|
||||
using op_t = cuda::std::plus<>;
|
||||
using init_value_t = T;
|
||||
|
||||
// Retrieve axis parameters
|
||||
const auto elements = static_cast<std::size_t>(state.get_int64("Elements{io}"));
|
||||
|
||||
thrust::device_vector<T> in = generate(elements);
|
||||
thrust::device_vector<T> out(1);
|
||||
|
||||
auto d_in = thrust::raw_pointer_cast(in.data());
|
||||
auto d_out = thrust::raw_pointer_cast(out.data());
|
||||
|
||||
// Enable throughput calculations and add "Size" column to results.
|
||||
state.add_element_count(elements);
|
||||
state.add_global_memory_reads<T>(elements, "Size");
|
||||
state.add_global_memory_writes<T>(1);
|
||||
|
||||
caching_allocator_t alloc;
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch& launch) {
|
||||
auto env = cub_bench_env(
|
||||
alloc,
|
||||
launch,
|
||||
cuda::execution::require(cuda::execution::determinism::not_guaranteed)
|
||||
#if !TUNE_BASE
|
||||
,
|
||||
cuda::execution::tune(policy_selector<cuda::std::__accumulator_t<op_t, T, init_value_t>>{})
|
||||
#endif // !TUNE_BASE
|
||||
);
|
||||
_CCCL_TRY_CUDA_API(
|
||||
cub::DeviceReduce::Reduce,
|
||||
"Reduce failed",
|
||||
d_in,
|
||||
d_out,
|
||||
static_cast<OffsetT>(elements),
|
||||
op_t{},
|
||||
init_value_t{},
|
||||
env);
|
||||
});
|
||||
}
|
||||
|
||||
#ifdef TUNE_T
|
||||
using value_types = nvbench::type_list<TUNE_T>;
|
||||
#else
|
||||
using value_types = nvbench::type_list<int32_t, int64_t, float, double>;
|
||||
#endif
|
||||
|
||||
NVBENCH_BENCH_TYPES(nondeterministic_sum, NVBENCH_TYPE_AXES(value_types, offset_types))
|
||||
.set_name("base")
|
||||
.set_type_axes_names({"T{ct}", "OffsetT{ct}"})
|
||||
.add_int64_power_of_two_axis("Elements{io}", nvbench::range(16, 28, 4));
|
||||
32
cccl_upstream/cub/benchmarks/bench/reduce/sum.cu
Normal file
32
cccl_upstream/cub/benchmarks/bench/reduce/sum.cu
Normal file
@@ -0,0 +1,32 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2011-2023, NVIDIA CORPORATION. All rights reserved.
|
||||
// SPDX-License-Identifier: BSD-3
|
||||
|
||||
// This benchmark is intended to cover redux instructions on Ampere+ architectures. It specifically uses
|
||||
// cuda::std::plus<> instead of a user-defined operator, which CUB recognizes to select an optimized code path.
|
||||
|
||||
// Tuning parameters found for signed integer types apply equally for unsigned integer types
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
// %RANGE% TUNE_ITEMS_PER_THREAD ipt 7:24:1
|
||||
// %RANGE% TUNE_THREADS_PER_BLOCK tpb 128:1024:32
|
||||
// %RANGE% TUNE_ITEMS_PER_VEC_LOAD_POW2 ipv 1:2:1
|
||||
|
||||
// __half and __nv_bfloat16 are added for full (non-tuning) runs; CUB has fast paths for them (see #9587).
|
||||
#ifdef TUNE_T
|
||||
using value_types = nvbench::type_list<TUNE_T>;
|
||||
#else
|
||||
using value_types =
|
||||
push_back_t<all_types
|
||||
# if _CCCL_HAS_NVFP16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
,
|
||||
__half
|
||||
# endif
|
||||
# if _CCCL_HAS_NVBF16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
,
|
||||
__nv_bfloat16
|
||||
# endif
|
||||
>;
|
||||
#endif
|
||||
using op_t = ::cuda::std::plus<>;
|
||||
#include "base.cuh"
|
||||
@@ -0,0 +1,41 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2025, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
|
||||
#pragma once
|
||||
|
||||
#include <cub/config.cuh>
|
||||
|
||||
#include <cuda_runtime_api.h>
|
||||
#include <device_side_benchmark.cuh>
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
struct benchmark_op_t
|
||||
{
|
||||
template <typename T>
|
||||
__device__ __forceinline__ T operator()(T thread_data) const
|
||||
{
|
||||
using WarpReduce = cub::WarpReduce<T>;
|
||||
using TempStorage = typename WarpReduce::TempStorage;
|
||||
__shared__ TempStorage temp_storage[32];
|
||||
auto warp_id = threadIdx.x / 32;
|
||||
return WarpReduce{temp_storage[warp_id]}.Reduce(thread_data, op_t{});
|
||||
}
|
||||
};
|
||||
|
||||
template <typename T>
|
||||
void warp_reduce(nvbench::state& state, nvbench::type_list<T>)
|
||||
{
|
||||
constexpr int block_size = 256;
|
||||
constexpr int unroll_factor = 128; // compromise between compile time and noise
|
||||
const auto& kernel = benchmark_kernel<block_size, unroll_factor, benchmark_op_t, T>;
|
||||
const int num_SMs = state.get_device().value().get_number_of_sms(); // NOLINT(bugprone-unchecked-optional-access)
|
||||
const int device = state.get_device().value().get_id(); // NOLINT(bugprone-unchecked-optional-access)
|
||||
int max_blocks_per_SM = 0;
|
||||
NVBENCH_CUDA_CALL_NOEXCEPT(cudaOccupancyMaxActiveBlocksPerMultiprocessor(&max_blocks_per_SM, kernel, block_size, 0));
|
||||
const int grid_size = max_blocks_per_SM * num_SMs;
|
||||
state.exec(nvbench::exec_tag::gpu | nvbench::exec_tag::no_batch, [&](nvbench::launch&) {
|
||||
kernel<<<grid_size, block_size>>>(benchmark_op_t{});
|
||||
});
|
||||
}
|
||||
|
||||
NVBENCH_BENCH_TYPES(warp_reduce, NVBENCH_TYPE_AXES(value_types)).set_name("base").set_type_axes_names({"T{ct}"});
|
||||
30
cccl_upstream/cub/benchmarks/bench/reduce/warp_reduce_min.cu
Normal file
30
cccl_upstream/cub/benchmarks/bench/reduce/warp_reduce_min.cu
Normal file
@@ -0,0 +1,30 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2025, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
// complex types cannot be compared with operator<
|
||||
using value_types = nvbench::type_list<
|
||||
int8_t,
|
||||
int16_t,
|
||||
int32_t,
|
||||
int64_t,
|
||||
#if _CCCL_HAS_INT128()
|
||||
int128_t,
|
||||
#endif
|
||||
#if _CCCL_HAS_NVFP16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
__half,
|
||||
#endif
|
||||
#if _CCCL_HAS_NVBF16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
__nv_bfloat16,
|
||||
#endif
|
||||
float,
|
||||
double
|
||||
#if _CCCL_HAS_FLOAT128()
|
||||
,
|
||||
__float128
|
||||
#endif
|
||||
>;
|
||||
|
||||
using op_t = ::cuda::minimum<>;
|
||||
#include "warp_reduce_base.cuh"
|
||||
35
cccl_upstream/cub/benchmarks/bench/reduce/warp_reduce_sum.cu
Normal file
35
cccl_upstream/cub/benchmarks/bench/reduce/warp_reduce_sum.cu
Normal file
@@ -0,0 +1,35 @@
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2025, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
|
||||
#include <nvbench_helper.cuh>
|
||||
|
||||
using value_types = nvbench::type_list<
|
||||
int8_t,
|
||||
int16_t,
|
||||
int32_t,
|
||||
int64_t,
|
||||
#if _CCCL_HAS_INT128()
|
||||
int128_t,
|
||||
#endif
|
||||
#if _CCCL_HAS_NVFP16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
__half,
|
||||
#endif
|
||||
#if _CCCL_HAS_NVBF16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
__nv_bfloat16,
|
||||
#endif
|
||||
float,
|
||||
double,
|
||||
#if _CCCL_HAS_FLOAT128()
|
||||
__float128,
|
||||
#endif
|
||||
#if _CCCL_HAS_NVFP16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
cuda::std::complex<__half>,
|
||||
#endif
|
||||
#if _CCCL_HAS_NVBF16() && _CCCL_CTK_AT_LEAST(12, 2)
|
||||
cuda::std::complex<__nv_bfloat16>,
|
||||
#endif
|
||||
cuda::std::complex<float>,
|
||||
cuda::std::complex<double>>;
|
||||
|
||||
using op_t = ::cuda::std::plus<>;
|
||||
#include "warp_reduce_base.cuh"
|
||||
Reference in New Issue
Block a user